//---------------------------------------------------------------------------------------------------------------------
//
// Project: ARMOSD
//
// Component: ARMOSD Board
//
// Started: September 2008
//
// Warning: No portion of the code used in this project may be distributed without the permission of all of the team members:
//          Didier Jaillant, Rolf Ziegler, Anthony Cake, Cedric Marzer
//
//softmodem.h

// soft_modem

int msk_counter;
int msk_type;
int msk_tone;


#define ASC1200 0
#define ASC1800 1
#define DESC1200 2
#define DESC1800 3

const unsigned long asc1200[12]={768,955,1024,955,768,512,256,69,0,69,256,512}; //starts ascending & ends ascending
const unsigned long asc1800[12]={874,1024,874,512,150,0,150,512,874,1024,874,512}; //starts ascending & ends descending
const unsigned long desc1200[12]={256,69,0,69,256,512,768,955,1024,955,768,512}; //starts descending & ends descending
const unsigned long desc1800[12]={150,0,150,512,874,1024,874,512,150,0,150,512}; //starts descending & ends ascending

void  Timer0Handler (void) __irq
{  
	unsigned long dac_value;
    T0IR = 1;			/* clear interrupt flag */
    IENABLE;			/* handles nested interrupt */

	#define DAC_BIAS	0x00010000
	
    if (msk_counter==0){
    	msk_tone=(IOPIN0 & (1L << (8)))>>8; //reading txd1 on pin 0.8
    	switch (msk_type){
	    	case ASC1200 : 
	    	
	    		if (msk_tone==1) msk_type=ASC1200;
	    		else msk_type=ASC1800;
	    		break;
	    	
	    	case  ASC1800 :
	    	
	    	    if (msk_tone==1) msk_type=DESC1200;
	    	    else msk_type=DESC1800;
	    	    break;
	    	
	    	case DESC1200 :
	    	
	    	    if (msk_tone==1) msk_type=DESC1200;
	    	    else msk_type=DESC1800; 
	    	    break;
	    	
	    	case DESC1800 :
	    	
	    	    if (msk_tone==1) msk_type=ASC1200;
	    	    else msk_type=ASC1800;    		    	
	    	    break;
    	}   	
    }
    
    switch (msk_type){
    	case ASC1200 :
    	
    		dac_value=asc1200[msk_counter];
    		break;
    	
    	case ASC1800 :
    	
    		dac_value=asc1800[msk_counter];
    		break;
    	
    	case DESC1200 :
    	
    		dac_value=desc1200[msk_counter];
    		break;
    	
    	case DESC1800 :
    	
    		dac_value=desc1800[msk_counter];
    		break;
    	
    }
    DACR = (dac_value << 6) | DAC_BIAS;
    msk_counter++;
    
    if (msk_counter==12) msk_counter=0;
    
    IDISABLE;
    VICVectAddr = 0;		/* Acknowledge Interrupt */
}

DWORD init_modem_timer (void) 
{
    T0MR0 = 1041;	/* the magic number */
    T0MCR = 3;			/* Interrupt and Reset on MR0 */
    if ( install_irq( TIMER0_INT, (void *)Timer0Handler ) == FALSE )
    {
	return (FALSE);
    }
    else
    {
	return (TRUE);
    }
}

void soft_modem_init(){
	PINSEL1 = 0x00080000;	/* set p0.25 to DAC output */
	msk_counter=0;
	msk_type=ASC1200;
	msk_tone=1;
	init_modem_timer();
	enable_timer( 0 );
}

void test_dac(){
	
	soft_modem_init();
	uart1Init(UART_BAUD(1200), UART_8N1, UART_FIFO_8); // setup the UART
		
	while(1){
		
		uart1Puts("\r\nhello world\r\n");
	}
}
